@@ -7,19 +7,16 @@ auc <- function(group, preds, roc = FALSE, cutoffs = NULL) {
   n0 <- n - n1
   s1 <- sum( ri[ group == 1 ] )
   auc <- ( s1 - 0.5 * n1 * (n1 + 1) ) / n0 / n1
-  result <- auc
+  result <- list(auc = auc)
   
   if ( roc ) {
-    
     if ( is.null(cutoffs) ) {
       lena <- seq(1, 0, by = -0.01)
-    } else  lena <- cutoffs
-    
+    } else  lena <- cutoffs  
     nu <- length(lena)
     sens <- spec <- numeric(nu)
     npos <- sum( group == 1 )
     nnegs <- sum( group == 0 )
-    
     for ( i in 1:nu ) {
       estp <- as.numeric( preds >= lena[i] )      
       sens[i] <-  sum( group == 1  &  estp == 1 ) / npos
@@ -31,9 +28,7 @@ auc <- function(group, preds, roc = FALSE, cutoffs = NULL) {
     sens[nu + 1] <- 1
     spec[nu + 1] <- 0
     cutoffs <- c(1, lena, 0)    
-
-    plot( 1 - spec, sens, type = "l", lty = 5, lwd = 2, xlab = "1- specificity", ylab = "Sensitivity", col = 3,
-          xlim = c(0, 1))
+    plot( 1 - spec, sens, type = "l", lty = 5, lwd = 2, xlab = "1- specificity", ylab = "Sensitivity", col = 3, xlim = c(0, 1) )
     abline(a = 0, b = 1, lty = 2, col = 2)
     qa <- which.max( sens + spec - 1 )
     points( 1 - spec[qa], sens[qa], lwd = 2)
@@ -43,5 +38,4 @@ auc <- function(group, preds, roc = FALSE, cutoffs = NULL) {
   }
   
   result
-  
 }
\ No newline at end of file
