@@ -4,20 +4,18 @@
 #####
 #####################
 #####################
-
 condi <- function(ind1, ind2, cs, dat, type = "pearson", rob = FALSE, R = 1) {
   ## ind1 and ind2 are the two indices of the two variables whose correlation is of interest
   ## cs is a vector with the indices of of variable(s),  over which the condition takes place
   ## dat is the data, a matrix form
   ## type is either "pearson" or "spearman"
-  ## For a robust estimation of the PEarson correlation set rob = TRUE or FALSE otherwise
+  ## For a robust estimation of the Pearson correlation set rob = TRUE or FALSE otherwise
   n <- dim(dat)[1] ## sample size
   d <- sum( cs>0 )  ## dimensionality of cs
-  
   if ( R == 1 ) {  ## no permutation
   ## NOTE: if you set test = "spearman", you must have the ranks of the data in 
   ## the dat argument and not the data themselves. This is to speed up the computations
-  if (type == "spearman") rob = FALSE   ## if spearman is chosen, robust is set to FALSE  
+  if (type == "spearman")  rob = FALSE   ## if spearman is chosen, robust is set to FALSE  
 
   if ( !rob ) {
     if ( d == 0 ) {
@@ -32,17 +30,16 @@ condi <- function(ind1, ind2, cs, dat, type = "pearson", rob = FALSE, R = 1) {
       ( solve( as.matrix( corrMatrix[csIdx, csIdx] ), rbind( corrMatrix[csIdx, xyIdx] ) ) ) 
       r <-  - residCorrMatrix[1, 2] / sqrt( residCorrMatrix[1, 1] * residCorrMatrix[2, 2]) 
       if ( abs(r) >1 )   r <- 0.99999
-      
     }
     
   } else {  ## robust estimation using M estimation
     if ( d == 0 ) {
-      b1 <- coef( MASS::rlm( dat[, ind1] ~ dat[, ind2], maxit = 2000 ) )[2]
-      b2 <- coef( MASS::rlm( dat[, ind2] ~ dat[, ind1], maxit = 2000 ) )[2]    
+      b1 <- coef( MASS::rlm( dat[, ind1] ~ dat[, ind2], maxit = 2000, method = "MM" ) )[2]
+      b2 <- coef( MASS::rlm( dat[, ind2] ~ dat[, ind1], maxit = 2000, method = "MM" ) )[2]    
       r <- sqrt( abs(b1 * b2) )
     } else {
-      e1 <- resid( MASS::rlm( dat[, ind1] ~.,  data = data.frame( dat[, c(ind2, cs) ] ), maxit = 2000 ) )
-      e2 <- resid( MASS::rlm( dat[, ind2] ~., data = data.frame( dat[, c(ind1, cs) ] ), maxit = 2000 ) )
+      e1 <- resid( MASS::rlm( dat[, ind1] ~.,  data = data.frame( dat[, c(ind2, cs) ] ), maxit = 2000, method = "MM") )
+      e2 <- resid( MASS::rlm( dat[, ind2] ~., data = data.frame( dat[, c(ind1, cs) ] ), maxit = 2000,method = "MM" ) )
       r <- cor(e1, e2)
     }
   }
@@ -64,10 +61,8 @@ condi <- function(ind1, ind2, cs, dat, type = "pearson", rob = FALSE, R = 1) {
       
       if ( rob ) { ## robust correlation
 
-        mod1 <- MASS::rlm( x1 ~ x2, maxit = 2000 )
-        mod2 <- MASS::rlm( x2 ~ x1, maxit = 2000 )       
-        e1 <- resid(mod1)
-        e2 <- resid(mod2)
+        e1 <- resid( MASS::rlm( x1 ~ x2, maxit = 2000, method = "MM" ) )
+        e2 <- resid( MASS::rlm( x2 ~ x1, maxit = 2000, method = "MM" ) )       
         res <- permcor( cbind(e1, e2), R )
         stat <- abs( res[1] )
         pvalue <- res[2]
@@ -78,20 +73,20 @@ condi <- function(ind1, ind2, cs, dat, type = "pearson", rob = FALSE, R = 1) {
         pvalue <- res[2]
       }
       
-    }else{  ## there are conditioning variables
+    } else {  ## there are conditioning variables
       
       if ( rob ) { ## robust correlation
-        e1 <- resid( MASS::rlm( x1 ~ ., data = as.data.frame(dat[, cs]) ), maxit = 2000 )
-        e2 <- resid( MASS::rlm( x2 ~.,  data = as.data.frame(dat[, cs]) ), maxit = 2000 )
+        e1 <- resid( MASS::rlm( x1 ~ ., data = as.data.frame(dat[, cs]), maxit = 2000, method = "MM") )
+        e2 <- resid( MASS::rlm( x2 ~.,  data = as.data.frame(dat[, cs]), maxit = 2000, method = "MM") )
         res <- permcor( cbind(e1, e2), R)
         stat <- abs( res[1] )
         pvalue <- res[2]
         
-      }else{
-        er <- resid( lm.fit(cbind(1, dat[, cs]), cbind( x1, x2 )  ) )
-        res <- permcor( er, R ) 
+      } else {
+        er <- lm.fit(cbind(1, dat[, cs]), cbind( x1, x2 )  )$residuals
+		res <- permcor( er, R ) 
         stat <- abs( res[1] )
-        pvalue <- res[2]
+        pvalue <- (res[2])
       }
     }
     #lets calculate the stat and p-value which are to be returned
