@@ -1,5 +1,4 @@
-fs.reg <- function(target, dataset, threshold = 0.05, wei = NULL, test = NULL, user_test = NULL, stopping = "BIC", tol = 2, robust = FALSE, ncores = 1) {
-  
+fs.reg <- function(target, dataset, ini = NULL, threshold = 0.05, wei = NULL, test = NULL, user_test = NULL, stopping = "BIC", tol = 2, robust = FALSE, ncores = 1) {
   ## target can be Real valued (normal), binary (binomial) or counts (poisson)
   ## dataset is a matrix or a data.frame with the predictor variables
   ## test is the test used, but not really required because depending on the data this will be decided.
@@ -13,24 +12,29 @@ fs.reg <- function(target, dataset, threshold = 0.05, wei = NULL, test = NULL, u
   ## robust is for robust modelling. TRUE or FALSE
   ## ncores is for parallel processing 
   
+  if ( !is.null(ini) ) {
+    result <- fs.reg_2(target, dataset, iniset = ini, wei = wei, threshold = threshold, tol = tol, robust = FALSE, ncores = ncores) 
+    
+  } else { 
+  
   threshold <- log(threshold)  ## log of the significance level
-  p <- ncol(dataset)  ## number of variables
+  dm <- dim(dataset)
+  p <- dm[2]  ## number of variables
   devi <- dof <- numeric( p )  
   moda <- list()
   k <- 1   ## counter
-  n <- length(target)  ## sample size
+  n <- dm[1]  ## sample size
   tool <- numeric( min(n, p) )
   result <- NULL
   con <- log(n)
   sela <- NULL
-  
   #check for NA values in the dataset and replace them with the variable median or the mode
-  if( any(is.na(dataset)) ) {
+  if ( any( is.na(dataset) ) ) {
     warning("The dataset contains missing values (NA) and they were replaced automatically by the variable (column) median (for numeric) or by the most frequent level (mode) if the variable is factor")
-    if (class(dataset) == "matrix")  {
+    if ( is.matrix(dataset) )  {
       dataset <- apply( dataset, 2, function(x){ x[which(is.na(x))] = median(x, na.rm = TRUE) ; return(x) } ) 
     } else {
-      poia <- which( is.na(dataset), arr.ind = TRUE )[2]
+      poia <- unique( which( is.na(dataset), arr.ind = TRUE )[, 2] )
       for( i in poia )  {
         xi <- dataset[, i]
         if ( is.numeric(xi) ) {                    
@@ -40,13 +44,9 @@ fs.reg <- function(target, dataset, threshold = 0.05, wei = NULL, test = NULL, u
       }
     }
   }
-  
   ##################################
   # target checking and initialize #
   ##################################
-  
-  if ( is.null( colnames(dataset) ) )    colnames(dataset) <- paste("X", 1:p, sep = "")
-  
   if ( is.null(test)  &  is.null(user_test) ) {
     
     ## surival data
@@ -62,18 +62,19 @@ fs.reg <- function(target, dataset, threshold = 0.05, wei = NULL, test = NULL, u
       } else  ci_test <- test <- "testIndReg"  
     }
   }
-  
-  #available conditional independence tests
-  av_models = c("testIndReg", "testIndBinom", "testIndBeta", "censIndCR", "testIndRQ", "censIndWR", "testIndLogistic", "testIndPois", "testIndNB", "testIndZIP", "testIndSpeedglm");
-  
+
   ci_test <- test
   #cat(test)
-  
   if ( ( test == "testIndLogistic" &  length( unique(target) ) == 2 )  ||  test == "testIndBinom"  || test == "testIndPois" ) {
     
     result <- glm.fsreg( target, dataset, wei = wei, threshold = exp(threshold), tol = tol, robust = robust, ncores = ncores) 
+
+  } else if ( test == "testIndFisher"  &  is.matrix(dataset)  &  is.null(wei) & is.null(ini) ) {
     
-  } else if ( test == "testIndReg"  &  !is.matrix(target) ) {
+	if (stopping == "adjrsq")  stopping = "ar2"
+    result <- Rfast::cor.fsreg(target, dataset, threshold = exp(threshold), tolb = tol, tolr = tol, stopping = stopping)
+    
+  } else if ( test == "testIndReg" || ( test == "testIndFisher"  &  !is.matrix(dataset) )  || (test == "testIndFisher"  &  !is.null(wei) ) ) {
     
     result <- lm.fsreg( target, dataset, wei = wei, threshold = exp(threshold), stopping = stopping, tol = tol, robust = robust, ncores = ncores ) 
   
@@ -84,6 +85,18 @@ fs.reg <- function(target, dataset, threshold = 0.05, wei = NULL, test = NULL, u
   } else if ( test == "testIndZIP" ) {
     
     result <- zip.fsreg(target, dataset, threshold = exp(threshold), wei = wei, tol = tol, ncores = ncores) 
+    
+  } else if ( test == "testIndGamma" ) {
+    
+    result <- gammafsreg(target, dataset, threshold = exp(threshold), wei = wei, tol = tol, ncores = ncores) 
+    
+  } else if ( test == "testIndNormLog" ) {
+    
+    result <- normlog.fsreg(target, dataset, threshold = exp(threshold), wei = wei, tol = tol, ncores = ncores) 
+    
+  } else if ( test == "testIndTobit" ) {
+    
+    result <- tobit.fsreg(target, dataset, threshold = exp(threshold), wei = wei, tol = tol, ncores = ncores) 
   
   } else if ( test == "testIndSpeedglm" ) {
     
@@ -95,9 +108,6 @@ fs.reg <- function(target, dataset, threshold = 0.05, wei = NULL, test = NULL, u
     
   } else {
     
-    test <- match.arg(test, av_models, TRUE);
-    #convert to closure type
-      
     if ( test == "censIndCR" ) {
       ci_test <- test
       test <- survival::coxph 
@@ -164,12 +174,10 @@ fs.reg <- function(target, dataset, threshold = 0.05, wei = NULL, test = NULL, u
     } else {
       cl <- makePSOCKcluster(ncores)
       registerDoParallel(cl)
-      mata <- matrix(0, p, 2)
       mod <- foreach( i = 1:p, .combine = rbind, .packages = c("MASS", "quantreg", "nnet", "survival", "ordinal") ) %dopar% {
         ww <- test( target ~ dataset[, i], weights = wei )
-        mata[i, ] <- c( 2 * as.numeric( logLik(ww) ), length( coef( ww ) )  )
+        return( c( 2 * as.numeric( logLik(ww) ), length( coef( ww ) ) ) )
       }
-      
       stopCluster(cl)
       
       stat <-  abs( mod[, 1] - ini )
@@ -180,7 +188,6 @@ fs.reg <- function(target, dataset, threshold = 0.05, wei = NULL, test = NULL, u
     mat <- cbind(1:p, pval, stat) 
     colnames(mat) <- c( "variables", "log.p-value", "stat" )
     rownames(mat) <- 1:p
-    
     sel <- which.min(pval)
     info <- matrix( numeric(3), ncol = 3 )
 
@@ -190,22 +197,19 @@ fs.reg <- function(target, dataset, threshold = 0.05, wei = NULL, test = NULL, u
       sela <- sel
       mi <- test( target ~ dataset[, sel], weights = wei )
       la <- logLik(mi)
-      tool[1] <-  - 2 * as.numeric( la ) +  attr(la, "df") * con
+      tool[1] <-  - 2 * as.numeric( la ) + attr(la, "df") * con
       moda[[ 1 ]] <- mi
     }  else  {
        info <- info  
        sela <- NULL
     }
-    
     ############
     ###       k equals 2
     ############ 
-    
     if ( info[k, 2] < threshold  &  nrow(mat) > 0 ) {
       
       k <- 2
       pn <- p - k + 1   
-
       ini <-  2 * as.numeric( la ) 
       do <- length( coef( moda[[ 1 ]]  ) ) 
       
@@ -216,7 +220,6 @@ fs.reg <- function(target, dataset, threshold = 0.05, wei = NULL, test = NULL, u
           devi[i] <-  2 * as.numeric( logLik(ww) )
           dof[i] <- length( coef( ww ) )          
         }
-        
         stat <- abs( devi - ini )
         pval <- pchisq( stat, dof - do, lower.tail = FALSE, log.p = TRUE )
         
@@ -224,19 +227,16 @@ fs.reg <- function(target, dataset, threshold = 0.05, wei = NULL, test = NULL, u
         
         cl <- makePSOCKcluster(ncores)
         registerDoParallel(cl)
-        mata <- matrix(0, pn, 2)  
         mod <- foreach( i = 1:pn, .combine = rbind, .packages = c("MASS", "quantreg", "nnet", "survival", "ordinal") ) %dopar% {
           ww <- test( target ~., data = as.data.frame( dataset[, c(sela, mat[i, 1]) ] ), weights = wei )
-          mata[i, ] <- c( 2 * as.numeric( logLik(ww) ), length( coef( ww ) )  )
+          return( c( 2 * as.numeric( logLik(ww) ), length( coef( ww ) ) ) )
         }
         stopCluster(cl)
-        
         stat <- abs( mod[, 1] - ini )
         pval <- pchisq( stat, mod[, 2] - do, lower.tail = FALSE, log.p = TRUE )
       }
     
     mat[, 2:3] <- cbind(pval, stat)
-    
     ina <- which.min(mat[, 2])
     sel <- mat[ina, 1]    
     
@@ -254,22 +254,16 @@ fs.reg <- function(target, dataset, threshold = 0.05, wei = NULL, test = NULL, u
         mat <- mat[-ina ,, drop = FALSE ] 
         moda[[ 2 ]] <- ma
       }
-      
     } else  info <- info  
-    
   }
-    
   ############
   ###       k greater than 2
   ############ 
-  
-  
   if ( nrow(info) > 1  &  nrow(mat) > 0 ) {
-    while ( ( info[k, 2] < threshold )  &  ( k < n )  &  ( tool[ k - 1 ] - tool[ k ] > tol )  &  nrow(mat) > 0 )  {
+    while ( info[k, 2] < threshold   & k < n - 15  & tool[ k - 1 ] - tool[ k ] > tol  &  nrow(mat) > 0 )  {
       
       ini =  2 * as.numeric( logLik( moda[[ k ]] ) ) 
       do = length( coef( moda[[ k ]]  ) ) 
-      
       k <- k + 1   
       pn <- p - k  + 1
       
@@ -287,20 +281,16 @@ fs.reg <- function(target, dataset, threshold = 0.05, wei = NULL, test = NULL, u
       } else {
           cl <- makePSOCKcluster(ncores)
           registerDoParallel(cl)
-          mata = matrix(0, pn, 2)  
           mod <- foreach( i = 1:pn, .combine = rbind, .packages = c("MASS", "quantreg", "nnet", "survival", "ordinal") ) %dopar% {
             ww <- test( target ~., data = as.data.frame( dataset[, c(sela, mat[i, 1]) ] ), weights = wei )
-            mata[i, ] <- c( 2 * as.numeric( logLik(ww) ), length( coef( ww ) )  )
+            return( c( 2 * as.numeric( logLik(ww) ), length( coef( ww ) ) ) )
           }
           stopCluster(cl)
-          
           stat = abs( mod[, 1] - ini )
           pval = pchisq( stat, mod[, 2] - do, lower.tail = FALSE, log.p = TRUE )
-          
         }
         
         mat[, 2:3] <- cbind(pval, stat)
-      
         ina <- which.min(mat[, 2])
         sel <- mat[ina, 1]    
         
@@ -320,42 +310,26 @@ fs.reg <- function(target, dataset, threshold = 0.05, wei = NULL, test = NULL, u
           } 
           
         } else   info <- rbind(info, c( 1e300, 0, 0 ) )
-      
     } 
-    
   }  
     
     runtime <- proc.time() - runtime
     
     d <- length(sela)
     final <- NULL
-    models <- NULL
-    
     if ( d >= 1 ) {
-      models <- NULL
-      xx <- as.data.frame( dataset[, sela] )
-      colnames(xx) <- paste("V", sela, sep = "") 
-      
-      if ( d == 1 ) {
-        models <- NULL
-        xx <- as.data.frame( dataset[, sela] )
-        colnames(xx) <- paste("V", sela, sep = "") 
-        models[[ 1 ]] <- test( target ~., data = data.frame( xx ), weights = wei )
-        
-      } else    for (i in 1:d)  models[[ i ]] <- test( target ~., data = data.frame( xx[, 1:i] ), weights = wei )
-      
-      final <- summary( models[[ d ]] )
-      info <- info[1:d, ]
-      if ( d == 1 )  info <- matrix(info, nrow = 1)
+      final <- test( target ~., data = as.data.frame( dataset[, sela] ), weights = wei )
+      info <- info[1:d,  , drop = FALSE]
       info <- cbind( info, tool[ 1:d ] ) 
       colnames(info) <- c( "variables", "log.p-values", "stat", "BIC" )
       rownames(info) <- info[, 1]
     }
     
-    result = list(mat = t(mat), info = info, models = models, final = final, ci_test = ci_test, runtime = runtime ) 
-    
+    result = list(runtime = runtime, mat = mat, info = info, ci_test = ci_test, final = final ) 
   }         
   
+  }
+  
   result
   
 }    
