@@ -1,4 +1,4 @@
-pc.or <- function(mod, graph = FALSE) {
+pc.or <- function(mod) {
   ## mod is the outcome of the function pc.con or pc.skel
   ## method is either "pearson", "spearman" or "cat"
   ## alpha is the threshold used in the skeleton of the pc
@@ -8,18 +8,19 @@ pc.or <- function(mod, graph = FALSE) {
   G <- mod$G  ## the undirected graph
   n <- ncol(G)  ## how many variables are there
   ina <- 1:n 
-  kappa <- mod$kappa
   sep <- mod$sepset  ## the triples with separating variables
   durat <-  proc.time()
   if ( length(sep) == 0 ) {
     G <- G 
     paste("There are no separating sets")
   } else {
-    
+ 
     len <- length(sep) 
     if ( len == 1 )  {
       if ( !is.matrix( sep[[ 1 ]] ) ) {
         sepa <- c( sep[[ 1 ]][ 1:3 ], sep[[ 1 ]][ 5 ] )
+      } else if ( dim( sep[[ 1 ]] )[1] == 1 ) {
+        sepa <- c( sep[[ 1 ]][ 1:3 ], sep[[ 1 ]][ 5 ] )
       } else {
         sepa <- cbind( sep[[ 1 ]][, 1:3 ], sep[[ 1 ]][, 5] )
       }
@@ -35,11 +36,11 @@ pc.or <- function(mod, graph = FALSE) {
           sep[[ i ]] <- c( sep[[ i ]][ 1:c(2 + i)], numeric( len - i ), sep[[ i ]][ 4 + i ] )
           sepa <- rbind( sepa, sep[[ i ]] ) 
         } else {
-          ante <- matrix( numeric( nrow( sep[[ i ]] ) * (len - i) ), ncol = len - i )
-          if ( sum( dim(ante) ) != 0 ) {
+          ante <- matrix(numeric( NROW( sep[[ i ]] ) * (len - i) ), ncol = len - i )
+          if ( sum( dim(ante) ) > 2 ) {
             la <- cbind( sep[[ i ]][, 1:c(2 + i) ], ante, sep[[ i ]][ , 4 + i ] )
           } else {
-            la <- cbind( sep[[ i ]][, 1:c(2 + i) ], sep[[ i ]][ , 4 + i ] )
+            la <- cbind( sep[[ i ]][, 1:c(2 + i), drop = FALSE ], numeric( len - i ), sep[[ i ]][ , 4 + i, drop = FALSE ] )
           }
           sepa <- rbind(sepa, la)
         }
@@ -48,73 +49,44 @@ pc.or <- function(mod, graph = FALSE) {
     
     if ( !is.matrix(sepa) )  sepa <- matrix( sepa, nrow = 1 )
     colnames(sepa) <- c("X", "Y", paste("Var", 1:len, sep = ""), "logged p-value")
-    m <- nrow(sepa)
-    p <- ncol(sepa) - 1
     rownames(sepa) <- colnames(G)[ sepa[, 1] ]
     ## sepa contains all the non significant pairs given at least one variable
     
     vim <- 1  ## 1st step	
     ### Orientation rule 0: find the triplets (v structures) and orient
-  	G <- R0(G, ina, sepa) 
-	
+    p <- dim(sepa)[2] - 1
+    G1 <- R0(G, ina, sepa[, 1:p, drop = FALSE]) 
     ## 1st orientation rule: if there is an arrow and then an edge make it arrow
-    G <- R1(G)
-	
+    G1 <- R1(G1)
     ## 2nd orientation rule: if there is a directed path connect the beginning with the end
-    G <- R2(G) 
-    
+    G1 <- R2(G1) 
     ### 3rd rule, the complicated one with v structures
-    G <- R3(G)
-    
-    G1 <- G
-    
+    G1 <- R3(G1)
     ## 1st orientation rule: if there is an arrow and then an edge make it arrow
-    G <- R1(G) 
-    
+    G2 <- R1(G1) 
     ## 2nd orientation rule: if there is a directed path connect the beginning with the end
-    G <- R2(G)    
-    
+    G2 <- R2(G2)    
     ### 3rd rule, the complicated one with v structures
-    G <- R3(G)
-    
-    G2 <- G
+    G2 <- R3(G2)
     
     vim <- 2 
     while ( sum( abs( G1 - G2 ) ) != 0 ) {
-      
 	  vim <- vim + 1 
       G1 <- G2   
       ## 1st orientation rule: if there is an arrow and then an edge make it arrow
-      G <- R1(G)
-      
+      G2 <- R1(G1)
       ## 2nd orientation rule: if there is a directed path connect the beginning with the end
-      G <- R2(G) 
-       
+      G2 <- R2(G2) 
       ### 3rd rule, the complicated one with v structures
-      G <- R3(G)
-	        
-      G2 <- G
+      G2 <- R3(G2)
     }
     
-    G <- G2
-    
   } ## end of all rules 
-  
-  
-  durat <- proc.time() - durat
-  colnames(G) <- rownames(G) <- colnames(mod$G)
-  
-  Ga <- G 
-  Ga[ Ga == 3 ] <- 0
-  
-  if ( graph ) {
     
-    plotnetwork(mod$G, titlos = paste("Skeleton of the PC algorithm for", mod$title ) )
-    dev.new()
-    plotnetwork(Ga, titlos = paste("CPDAG of the PC algorithm for", mod$title ))
-  }
+  durat <- proc.time() - durat
+  colnames(G2) <- rownames(G2) <- colnames(mod$G)
   
-  final = list(Gini = mod$G, G = G, runtime = durat) 
+  final = list(Gini = mod$G, G = G2, runtime = durat) 
   final
 }
 
@@ -132,7 +104,6 @@ pc.or <- function(mod, graph = FALSE) {
 
 R0 <- function(G, ina, sepa) {
   
-    p <- ncol(sepa) - 1
     l <- Rfast::rowsums( G == 1 )
     id <- which( l >= 0 ) 
     
@@ -142,25 +113,26 @@ R0 <- function(G, ina, sepa) {
         adj <- ina[ G[i, ] == 1 ]
         if ( length(adj) > 1 ) {
           sam <-  as.matrix(  t( combn(adj, 2) ) )
-          ela <- NULL
           for ( j in 1:nrow(sam) ) {
             if ( G[sam[j, 1], sam[j, 2] ] == 0 ) {
-              res <- is.sepset( sam[j, ], i, sepa[, 1:p] )
+              res <- is.sepset( sam[j, ], i, sepa )
               if ( !res ) {
-                G[ sam[j, 1], i ] = G[ sam[j, 2], i ] = 2 
-                G[ i, sam[j, 1] ] = G[ i, sam[j, 2] ] = 3
-                
-              } else G <- G  
-			  
+                if ( G[ sam[j, 1], i ] == 1 ) { 
+                  G[ sam[j, 1], i ] = 2
+                  G[ i, sam[j, 1] ] = 3
+                } 
+                if ( G[ sam[j, 2], i ] == 1 ) {   
+                  G[ sam[j, 2], i ] = 2 
+                  G[ i, sam[j, 2] ] = 3
+                }
+              }   
             }
           }
         }
-      }
-	  
+      } 
+
     }
-	
-	G
-	
+  G	
 }
     
 	
@@ -185,37 +157,24 @@ R1 <- function(G) {
   G
 }   
 
-
 R2 <- function(G) {
 
    if ( sum( G == 2 ) > 0  &  sum( G == 1 ) > 0 ) {
-      
-      poia <- which( G == 1, arr.ind = TRUE ) 
-      poia1 <- which( G == 1 )
-      poia2 <- which( G == 3 )
-      GGG <- G
-      GGG[poia1] <- NA
-      GGG[poia2] <- NA
-      GGG[ GGG == 0 ] <- NA
-      g1 <- e1071::allShortestPaths(GGG)
-      nu <- nrow(poia)
-      
-      if ( nu > 0 ) {
-        for ( i in 1:nrow(poia) )  {
-          aa <- e1071::extractPath( g1, start = poia[i, 1], end = poia[i, 2] )
-          if ( length(aa) > 2 ) {
-            if ( G[ poia[i, 1], poia[i, 2] ] == 1 ) {
-              G[ poia[i, 1], poia[i, 2] ] <- 2
-              G[ poia[i, 2], poia[i, 1] ] <- 3
-            }
-          }
-        }   
+      tup <- which( G == 2, arr.ind = TRUE )
+      nup <- nrow(tup) 
+      for (i in 1:nup) {
+        arxi <- tup[i, 1] 
+        can <- tup[i, 2] 
+        geit <- which( G[can, ] == 2 )
+        if ( sum( G[arxi, geit] == 1 ) > 0 ) {
+          G[arxi, geit] <- 2
+          G[geit, arxi] <- 3
+        }  
       }
-      
    } 
 	
-   G
-}	
+  G
+} 
 
 
 R3 <- function(G) {
@@ -254,11 +213,13 @@ R3 <- function(G) {
 }
 
 
+
+
 is.sepset <- function(pair, nd, separ) {
-  
-  a <- which( Rfast::colsums(t(separ[, 1:2]) - pair ) == 0 )
-  if ( length(a) > 1 )  { 
-    b <- length( intersect(nd, separ[a, -c(1:2)]) ) > 0
+  pair <- sort(pair)
+  a <- which( Rfast::colsums( abs( t(separ[, 1:2, drop = FALSE]) - pair ) ) == 0 )
+  if ( length(a) >= 1 )  { 
+    b <- length( intersect(nd, separ[a, -c(1:2), drop = FALSE]) ) > 0
   } else b <- FALSE
   
   b
